@@ -39,6 +39,7 @@ class DataSchemaBuilder:
         self._serialization_mode = None  # type: typing.Optional[SerializationMode]
         self._is_union = False
         self._bit_length_computed_at_least_once = False
+        self._doc = ""
 
     @property
     def fields(self) -> typing.List[_serializable.Field]:
@@ -57,6 +58,10 @@ class DataSchemaBuilder:
         out += self.constants
         return out
 
+    @property
+    def doc(self) -> str:
+        return self._doc
+
     @property
     def serialization_mode(self) -> typing.Optional[SerializationMode]:
         return self._serialization_mode
@@ -80,6 +85,9 @@ class DataSchemaBuilder:
         assert isinstance(out, _bit_length_set.BitLengthSet) and len(out) > 0
         return out
 
+    def set_comment(self, comment: str) -> None:
+        self._doc = comment
+
     def add_field(self, field: _serializable.Field) -> None:
         if self.union and self._bit_length_computed_at_least_once:
             # Refer to the DSDL specification for the background information.
